/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

/* USER INFORMATION FUNCTIONS
 */


#include "backtalk.h"

#include <pwd.h>

#include "btpasswd.h"
#include "sysdict.h"
#include "last_log.h"
#include "stack.h"
#include "free.h"
#include "file.h"
#include "dict.h"
#include "user.h"
#include "userfunc.h"
#include "udb.h"
#include "group.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

extern char *effective_user;


/* GETUSERDATA - Load n fields of user data.  The field names are given in the
 * array of Tokens pointed to by 'varray'.  If 'val' is not NULL, the values
 * will be stored there.  Otherwise, they will be stored in the system
 * dictionary.  Returns non-zero if the user does not exist.
 */

int getuserdata(char *name, Array *varray, Token *val)
{
    int mine, i, gotit, needmore= FALSE;
    struct btpasswd *pw;
    extern struct btpasswd *upw;
    Token *cv;

    if (name[0] == '\0')
    {
	if (effective_user != NULL)
	{
	    if ((pw= getdbnam(effective_user)) == NULL)
		die("Select user %s does not exist",effective_user);
	    mine= TRUE;
	}
	else
	{
	    if (upw == NULL) return 1;
	    pw= upw;
	    mine= TRUE;
	}
	name= pw->name;
    }
    else
    {
	pw= getdbnam(name);
	if (pw == NULL) return 1;
	mine= ((idict(VAR_UID) == pw->uid) || bdict(VAR_AMADM) ||
		(user_gradm() && idict(VAR_GID) == pw->gid));
    }

    if (val == NULL)
	cv= (Token *) malloc(sizeof(Token));

    /* First handle all 'special' fields */
    for (i= 0; i < varray->sz; i++)
    {
	gotit= FALSE;

	if (val != NULL)
	    cv= val+i;

	/* Special names should never be bound */
	if (type(varray->a[i]) == TK_BOUND_LITERAL) continue;

	if (!strcmp(sval(varray->a[i]),"usrid"))
	{
	    cv->flag= TK_INTEGER;
	    cv->val= (void *)pw->uid;
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"usrlogin"))
	{
	    cv->flag= TK_STRING | TKF_FREE;
	    cv->val= (void *)strdup(pw->name);
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"grpid"))
	{
	    cv->flag= TK_INTEGER;
	    cv->val= (void *)pw->gid;
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"usrname"))
	{
	    cv->flag= TK_STRING | TKF_FREE;
	    if (bdict(VAR_MAYSEEFNAME) || mine)
		cv->val= (void *)strdup(pw->gecos);
	    else
		cv->val= (void *)strdup("");
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"dirpath"))
	{
	    cv->flag= TK_STRING | TKF_FREE;
	    cv->val= (void *)strdup(pw->dir);
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"laston"))
	{
	    cv->flag= TK_TIME;
	    cv->val= (void *)get_lastlog(pw->uid);
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"usrstat"))
	{
	    char *pass= getpassword(name,FALSE);
	    cv->flag= TK_INTEGER;
	    cv->val= (void *)((pass == NULL || pass[0] == '*') ? 2 :
				 (pass[0] == '?'));
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"email") && pw->email != NULL)
	{
	    cv->flag= TK_STRING | TKF_FREE;
	    cv->val= (void *)strdup((pw->epriv != 0 || mine) ? pw->email : "");
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"evalid") && pw->valcode != NULL)
	{
	    cv->flag= TK_STRING | TKF_FREE;
	    cv->val= (void *)strdup(pw->valcode);
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"privacy") && pw->priv >= 0)
	{
	    cv->flag= TK_INTEGER;
	    cv->val= (void *)(int)pw->priv;
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"eprivacy") && pw->epriv >= 0)
	{
	    cv->flag= TK_INTEGER;
	    cv->val= (void *)(int)pw->epriv;
	    gotit= TRUE;
	}
	else if (!strcmp(sval(varray->a[i]),"passwd") && pw->epriv >= 0)
	{
	    cv->flag= TK_STRING | TKF_FREE;
	    cv->val= (void *)malloc(1);
	    *sval(*cv)= '\0';
	    gotit= TRUE;
	}

	if (gotit)
	{
	    /* Don't load from .backtalk */
	    varray->a[i].sfid= 0;

	    /* Store in dictionary if that's what we are doing */
	    if (val == NULL)
		store_dict(sval(varray->a[i]),cv,FALSE);
	}
	else if (pw->priv == 1 && !mine)
	{
	    /* Can't read .backtalk, so don't have a value */
	    cv->flag= TK_STRING | TKF_FREE;
	    cv->val= (void *)strdup("");
	}
	else
	{
	    /* Mark for load from .backtalk */
	    varray->a[i].sfid= 1;
	    needmore= TRUE;
	}
    }

    if (val == NULL) free(cv);

    if (needmore && (pw->priv != 1 || mine))
	btloadvar(pw->name, varray, val);

    return 0;
}


/* INTVAL - Return the value of a token as an integer, converting strings to
 * integers, if possible.  Returns non-zero if the conversion failed.
 * Otherwise sets val.
 */
int intval(Token *tok, int *val)
{
    if (type(*tok) == TK_INTEGER)
    {
	*val= ival(*tok);
	return 0;
    }
    else if (type(*tok) == TK_STRING)
    {
	char *str= sval(*tok);
	if (isascii(str[0]) && isdigit(str[0]))
	{
	    *val= atoi(str);
	    return 0;
	}
    }
    return 1;
}


/* SAVEUSERDATA - Save user data.  The field names are given in the
 * array of Tokens pointed to by 'varray'.  If 'val' is not NULL, the values
 * will be taken from there.  Otherwise, they will be taken from system
 * dictionary values.  If err is non-NULL, we return an error message
 * in that BFSZ-sized buffer for bad values.  Otherwise, we just die with
 * such errors.
 */

int saveuserdata(char *name, Token *vt, Token *val, char *err)
{
    int i, savemore= FALSE;
    int myerr, amadm;
    struct btpasswd *pw;
    extern struct btpasswd *upw;
    Token *cv;
    int set_priv= FALSE, set_gid= FALSE, set_passwd= FALSE;
    int set_email= FALSE, set_fname= FALSE;
    int new_priv, new_epriv, new_gid, new_status, new_eval;
    char *new_passwd, *new_fname, *new_email, *uname;
    Array *varray= aval(*vt);

    /* Prepare the error buffer */
    if (myerr= (err == NULL))
	err= (char *)malloc(BFSZ);
    err[0]= '\0';

    /* Find the user */
    if (name[0] == '\0')
    {
	if (effective_user != NULL)
	{
	    /* Use effective user */
	    if ((pw= getdbnam(effective_user)) == NULL)
		die("Selected user %s does not exist",effective_user);
	    amadm= TRUE;
	}
	else
	{
	    /* Use current user */
	    if (upw == NULL)
		die("No user logged in");
	    pw= upw;
	    amadm= bdict(VAR_AMADM);
	}
	uname= strdup(pw->name);
    }
    else
    {
	/* Use named user */
	pw= getdbnam(uname= name);
	if (pw == NULL) die("named user %s does not exist",name);
	if (idict(VAR_UID) == pw->uid)
	    amadm= bdict(VAR_AMADM);
	else
	{
	    check_adm_or_gradm(pw);
	    amadm= TRUE;
	}
    }

    /* First dig up new values  for all 'special' fields */
    for (i= 0; i < varray->sz; i++)
    {
	/* Special names should never be bound */
	if (type(varray->a[i]) == TK_BOUND_LITERAL) continue;

	if (!strcmp(sval(varray->a[i]),"usrid") ||
	    !strcmp(sval(varray->a[i]),"usrlogin") ||
	    !strcmp(sval(varray->a[i]),"dirpath") ||
	    !strcmp(sval(varray->a[i]),"laston"))
	{
	    /* These things cannot be editted */
	    die("Cannot change value of key %s",varray->a[i]);
	}
	else if (!strcmp(sval(varray->a[i]),"grpid"))
	{
#if NOPWEDIT
	    die("Cannot change primary group ids on real unix accounts");
#else
	    if (!bdict(VAR_AMADM))
		die("Only conference administrators can change grpid");
	    set_gid= TRUE;
	    cv= (val == NULL) ? get_dict("grpid") : &val[i];
	    if (cv == NULL)
		die("variable grpid not defined");
	    if (type(*cv) == TK_INTEGER)
	    {
		new_gid= ival(*cv);
		if (new_gid < 0)
		    strcpy(err,"variable grpid may not be negative");
	    }
	    else if (type(*cv) == TK_STRING)
	    {
		char *s= sval(*cv);
		if (s[0] == '0' && s[0] <= '9')
		    new_gid= atoi(s);
		else if ((new_gid= groupid(s)) < 0)
		    sprintf(err,"Group %.100s does not exist",s);
	    }
	    else
		strcpy(err,"variable grpid should be string or integer");
	    varray->a[i].sfid= 2;
#endif /* NOPWEDIT */
	}
	else if (!strcmp(sval(varray->a[i]),"usrname"))
	{
#if NOPWEDIT
	    die("Cannot change fullnames on real unix accounts");
#else
	    set_fname= TRUE;
	    cv= (val == NULL) ? get_dict("usrname") : &val[i];
	    if (cv == NULL)
		die("variable usrname not defined");
	    if (type(*cv) != TK_STRING)
		die("usrname value is not a string");
	    new_fname= sval(*cv);
	    fullname_bad(new_fname,err);
	    varray->a[i].sfid= 2;
#endif /* NOPWEDIT */
	}
	else if (!strcmp(sval(varray->a[i]),"passwd"))
	{
#if NOPWEDIT
	    die("Cannot change passwords on real unix accounts");
#else
	    set_passwd= TRUE;
	    new_status= 0;
	    cv= (val == NULL) ? get_dict("passwd") : &val[i];
	    if (cv == NULL)
		die("variable passwd not defined");
	    if (type(*cv) != TK_STRING)
		die("passwd value is not a string");
	    new_passwd= sval(*cv);
	    passwd_bad(new_passwd,pw->name,err);
	    varray->a[i].sfid= 2;
#endif /* NOPWEDIT */
	}
	else if (!strcmp(sval(varray->a[i]),"usrstat"))
	{
#if NOPWEDIT
	    die("Cannot change validation status of real unix accounts");
#else
	    if (!amadm)
		die("Only administrators can change usrstat");
	    if (!set_passwd) new_passwd= NULL;
	    set_passwd= TRUE;
	    cv= (val == NULL) ? get_dict("usrstat") : &val[i];
	    if (cv == NULL)
		die("variable usrstat not defined");
	    if (intval(cv,&new_status))
		die("usrstat value is not an integer");
	    if (new_status < 0 || new_status > 2)
		strcpy(err, "userstat value must be 0, 1, or 2");
	    varray->a[i].sfid= 2;
#endif /* NOPWEDIT */
	}
	else if (!strcmp(sval(varray->a[i]),"email") && save_more())
	{
	    if (!set_email) new_eval= (idict(VAR_VALIDATEMAIL) == 0);
	    set_email= TRUE;
	    cv= (val == NULL) ? get_dict("email") : &val[i];
	    if (cv == NULL)
		die("variable email not defined");
	    if (type(*cv) != TK_STRING)
		die("email value is not a string");
	    new_email= sval(*cv);
	    varray->a[i].sfid= 2;
	}
	else if (!strcmp(sval(varray->a[i]),"evalid") && save_more())
	{
	    if (!amadm)
		die("Only administrators can change evalid");
	    if (!set_email) new_email= pw->email;
	    set_email= TRUE;
	    cv= (val == NULL) ? get_dict("evalid") : &val[i];
	    if (cv == NULL)
		die("variable evalid not defined");
	    if (type(*cv) != TK_STRING)
		die("evalid value is not a string");
	    if (*sval(*cv) != '\0')
		strcpy(err, "evalid key can only be set to an empty string");
	    new_eval= 1;
	    varray->a[i].sfid= 2;
	}
	else if (!strcmp(sval(varray->a[i]),"privacy") && save_more())
	{
	    if (!set_priv) new_epriv= pw->epriv;
	    set_priv= TRUE;
	    cv= (val == NULL) ? get_dict("privacy") : &val[i];
	    if (cv == NULL)
		die("variable privacy not defined");
	    if (intval(cv,&new_priv))
		die("privacy value is not an integer");
	    if (new_priv < 0 || new_priv > 1)
		strcpy(err, "privacy value must be 0 or 1");
	    varray->a[i].sfid= 2;
	}
	else if (!strcmp(sval(varray->a[i]),"eprivacy") && save_more())
	{
	    if (!set_priv) new_priv= pw->priv;
	    set_priv= TRUE;
	    cv= (val == NULL) ? get_dict("eprivacy") : &val[i];
	    if (cv == NULL)
		die("variable eprivacy not defined");
	    if (intval(cv,&new_epriv))
		die("eprivacy value is not an integer");
	    varray->a[i].sfid= 2;
	}
	else
	{
	    /* Save it in .backtalk */
	    varray->a[i].sfid= 0;
	    savemore= TRUE;
	}

	if (err[0] != '\0')
	{
	    if (myerr)
		die(err);
	    else
		return 1;
	}
    }

    /* Change email, if necessary */
    if (set_email)
	change_email(pw,new_email,new_eval);

    /* Not safe to use 'pw' variable after this since any of these
     * 'change' routines may do a flushusercache() which may invalidate
     * the pw variable.  Instead use getdbnam() which will get the pw
     * structure from cache if it is still there.
     */

    /* Change privacy, if necessary */
    if (set_priv)
	change_privacy(getdbnam(uname),new_priv,new_epriv);

#if !NOPWEDIT
    /* Change fullname, if necessary */
    if (set_fname)
	change_fname(getdbnam(uname),new_fname);

    /* Change group, if necessary */
    if (set_gid)
	change_gid(getdbnam(uname),new_gid);

    /* Change password/validation, if necessary */
    if (set_passwd)
    {
	char newpw[BFSZ], *npw= newpw;
	if (new_status > 0)
	    *(npw++)= (new_status == 1) ? '?' : '*';

	if (new_passwd != NULL)
	    strcpy(npw, mkpasswd(new_passwd,0));
	else if ((new_passwd= getpassword(uname, FALSE)) != NULL)
	{
	    while (*new_passwd == '?' || *new_passwd == '*')
		new_passwd++;
	    strcpy(npw, new_passwd);
	}
	else 
	    die("Cannot get old encrypted password");
	change_passwd(getdbnam(uname),newpw);
    }
#endif /* NOPWEDIT */

    if (savemore)
	btsavevar(uname ,varray, val);

    /* If we altered current user, reload upw variable */
    if ((set_email || set_priv || set_fname || set_gid || set_passwd) &&
	!strcmp(uname, upw->name))
    {
	struct btpasswd *tmp= dup_passwd(getdbnam(upw->name));
	free_passwd(upw);
	upw= tmp;
    }

    if (myerr) free(err);
    if (uname != name) free(uname);
    return 0;
}


/* FUNC_PUSHUSER:  <id> <varlist> pushuser <values>... 0
 *                 <id> <varlist> pushuser 1
 *
 * Push the user data values for the fields named in varlist onto the stack.
 * If <id> is an empty string, this works for the current users.  Varlist can
 * be a single literal or an array of literals.  If the user does not exist,
 * just a 1 is pushed.
 *
 * Values are taken either from the identity database, if defined there,
 * or from the user's .backtalk file.  The following values may be taken from
 * the identity database:
 *
 *   /usrid       User ID number
 *   /usrlogin    User login name
 *   /grpid       Primary group ID number
 *   /usrname     User's full name
 *   /dirpath     User's home directory path
 *   /laston      User's last login date/time.
 *   /usrstat     Status: 0=valid, 1=unvalidated, 2=invalidated, -1=nonexistant
 *   /email       User's email address.
 *   /evalid      If email unvalidated, validation string.  If valid, ().
 *   /privacy     User's privacy setting.
 *   /eprivacy    User's email privacy setting.
 *
 * If any other names are passed in, their values are fetched from the user's
 * .backtalk file.  If the current user does not match the named user, and the
 * current user is not an administrator, many values may be unavailable,
 * depending on the privacy flag.
 *
 * The obsolete 'userinfo' call is replaced by
 *   <login> [/usrid /grpid /usrname /dirpath /laston /usrstat] pushuser pop
 */

void func_pushuser()
{
    char *name;
    Token vt;
    Token *values;
    int i, rc;

    pop_litarray(&vt);
    name= pop_string();

    values= (Token *) malloc(aval(vt)->sz * sizeof(Token));

    if (getuserdata(name, aval(vt), values))
	push_integer(1);
    else
    {
	/* Push the result values */
	for (i= 0; i < aval(vt)->sz; i++)
	{
	    if (aval(vt)->a[i].sfid != 1)
		/* Got a value for the token */
		push_token(&values[i],FALSE);
	    else
		/* Push default */
		push_string(NULL,TRUE);
	}

	/* Push the success flag */
	push_integer(0);
    }


    free(values);
    free_val(&vt);
    free(name);
}


/* FUNC_LOADUSER:  <id> <varlist> pushuser <err>
 *
 * Just like pushuser, except instead of pushing the values assocated with
 * the listed identifiers, we load them into the current dictionary, in the
 * manner of loadvar.  If <id> is an empty string, this works for the current
 * user.  Varlist can be a single literal or an array of literals.  If the
 * user does not exist, an error code of 1 is pushed.
 */

void func_loaduser()
{
    char *name;
    Token vt;

    pop_litarray(&vt);
    name= pop_string();

    push_integer(getuserdata(name, aval(vt), NULL));

    free_val(&vt);
    free(name);
}


/* FUNC_SAVEUSER:  <id> <varlist> saveuser -
 *
 * Save new values for the named keys.  Values are taken from variables by the same
 * name.  Dies on errors.
 */

void func_saveuser()
{
    char *name;
    Token vt;

    pop_litarray(&vt);
    name= pop_string();

    saveuserdata(name, &vt, NULL, NULL);

    free_val(&vt);
    free(name);
}


/* FUNC_DEFUSER:  <login> <key> <value> defuser <err>
 *
 * Save a single new value for a single key.  Returns a text string which is null
 * if there is no error, and an error message otherwise.
 */

void func_defuser()
{
    char *name;
    Token label, value;
    Token ta;
    Array a;
    char err[BFSZ];

    pop_any(&value);
    pop_literal(&label);
    name= pop_string();

    ta.flag= TK_ARRAY;
    ta.val= (void *)&a;
    a.lk= a.sz= 1;
    a.a= &label;

    if (saveuserdata(name, &ta, &value, err))
	push_string(err, TRUE);
    else
	push_string(NULL, TRUE);

    free_val(&value);
    free_val(&label);
    free(name);
}


/* FUNC_VALCODEL:  <login> <email> <valcode> valcode <err>
 * Attempt to validate a user's email address, by entering the validation
 * code.  If the validation code is correct, the email address is validated
 * and we return 0.  If the address is already validated, then do nothing,
 * and also return 0.  Otherwise we return the following error codes:
 *   1 = no such user
 *   2 = email address does not match current saved one.
 *   3 = validation code does not match current saved one.
 * If we are automatically validating accounts once the email address is
 * validated, then this does that too.
 */

void func_valcode()
{
    struct btpasswd *pw;
    char *vc= pop_string();
    char *em= pop_string();
    char *us= pop_string();

    /* Find user */
    pw= getdbnam(us);
    free(us);
    if (pw == NULL)
    {
	free(vc); free(em);
	push_integer(1);
	return;
    }

    /* Check if email address matches */
    if (pw->email == NULL || strcasecmp(em,pw->email))
    {
	free(vc); free(em);
	push_integer(2);
	return;
    }

    /* If already validated, be happy */
    if (pw->valcode == NULL || pw->valcode[0] == '\0')
    {
	free(vc); free(em);
	push_integer(0);
	return;
    }

    /* Check if validation code matches */
    if (strcmp(pw->valcode, vc))
    {
	free(vc); free(em);
	push_integer(3);
	return;
    }
    free(vc);

    /* Validate */
    change_email(pw, em, 1);
    /* Can't use pw after this point - it's probably been deallocated */

    if (idict(VAR_VALIDATEMAIL) == 2)
    {
	char *pass= getpassword(us,FALSE);
	if (pass != NULL && pass[0] == '?')
	    change_passwd(getdbnam(us),pass+1);
    }

    free(em);
    push_integer(0);
    return;
}


/* FUNC_FIRSTUSER: - firstuser <username>
 * This returns the name of the "first" user in the user database.  It can
 * be used, together with "nextuser" to walk through the list of users.  If
 * there are no users, an empty string is pushed.
 */

void func_firstuser()
{
    push_string(firstusr(),TRUE);
}


/* FUNC_NEXTUSER: - nextuser <username>
 * This returns the name of the "next" user in the user database.  It can
 * be used, together with "firstuser" to walk through the list of users.  If
 * there are no more users, an empty string is pushed.
 */

void func_nextuser()
{
    push_string(nextusr(),TRUE);
}


/* FUNC_SEEKUSER: <username> seekuser -
 * This sets the user database so that the next call to "nextuser" will return
 * the user *after* the named user.  It is somewhat slow on some types of
 * databases.
 */

void func_seekuser()
{
    char *login= pop_string();
    seekusr(login);
    free(login);
}


/* FUNC_GROUPNAME: <gid> groupname <groupname>
 *
 * Return the group name associated with the given group ID number.
 */

void func_groupname()
{
    push_string(groupname(pop_integer()),TRUE);
}


/* FUNC_GROUPID: <groupname> groupid <gid>
 *
 * Return the gid number associated with the given group name.
 */

void func_groupid()
{
    char *gname;

    gname= pop_string();
    if (gname[0] >= '0' && gname[0] <= '9')
    	push_integer(atoi(gname));
    else
	push_integer(groupid(gname));
    free(gname);
}


/* FUNC_INGROUP: <group> <userid> ingroup <boolean>
 *
 * Return true if given user is in the given group.  If the user is (), then
 * the currently logged in user is checked.  The group can be given as a name
 * or number, and the number can be in integer or string format.
 */

void func_ingroup()
{
    Token tk;
    char *ulogin= pop_string();
    int fu= 1;
    struct btpasswd *pw;
    gid_t ugid;

    pop_any(&tk);

    if (*ulogin == '\0')
    {
	if (idict(VAR_UID) < 0 || sdict(VAR_ID)[0] == '\0')
	{
	    /* No current user */
	    push_integer(0);
	    free(ulogin);
	    free_val(&tk);
	    return;
	}
        else
	{
	    /* Check current user */
	    free(ulogin);  fu= 0;
	    ulogin= user_id();
	    ugid= user_gid();
	}
    }
    else
    {
    	/* Check given user */
	pw= getdbnam(ulogin);
	if (pw == NULL)
	{
	    /* No such user */
	    push_integer(0);
	    if (fu) free(ulogin);
	    free_val(&tk);
	    return;
	}
	ugid= pw->gid;
    }

    if (type(tk) == TK_INTEGER)
	push_integer(ingroupno(ival(tk),ugid,ulogin));
    else if (type(tk) == TK_STRING)
    {
	char *s= sval(tk);
	if (s[0] >= '0' && s[0] <= '9')
	    push_integer(ingroupno(atoi(s),ugid,ulogin));
	else
	    push_integer(ingroupname(s,ugid,ulogin));
    }
    else
    	die("expected string or integer argument");

    if (fu) free(ulogin);
    free_val(&tk);
}


/* FUNC_GROUPS: - groups <groupname1> <groupname2> ...
 *
 * Push all known group names on the stack.
 */

void push_str_copy(char *name) { push_string(name,TRUE); }

void func_groups()
{
    for_groups(&push_str_copy);
}


/* FUNC_DFLTGROUP: - dfltgroup <groupname>
 *
 * Push the name of the default user group.  Pushs () if there is none.
 */

void func_dfltgroup()
{
    char *g= user_group_name();

    push_string(g ? g : "",TRUE);
}


/* FUNC_NEWGROUP:   <groupname> newgroup <flag>
 *   Creates a new group with the given name.  Returns 0 on success, 1 if
 *   the group already exists.
 */

void func_newgroup()
{
#if NOPWEDIT
    die("Group creation not enabled in this installation");
#else
    char *gname= pop_string();

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may create new groups");

    push_integer(newgroup(gname));

    free(gname);
#endif /*NOPWEDIT*/
}


/* FUNC_NEWGROUPMEM:   <gname> <member> newgroupmem <flag>
 *   Add a user to an existing group.  Returns 0 on success, 1 if the group
 *   does not exists.
 */

void func_newgroupmem()
{
#if NOPWEDIT
    die("Changing group membership not enabled in this installation");
#else
    char *mem= pop_string();
    char *gname= pop_string();

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may add users to groups");

    push_integer(newgroupmem(gname,mem));

    free(mem);
    free(gname);
#endif /*NOPWEDIT*/
}


/* FUNC_DELGROUP:   <groupname> delgroup <flag>
 *   Delete a group and all its members.
 */

void func_delgroup()
{
#if NOPWEDIT
    die("Group deletion not enabled in this installation");
#else
    char *gname= pop_string();
    struct grp *groups;

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may remove groups");

    push_integer(delgroup(gname));

    free(gname);
#endif /*NOPWEDIT*/
}


/* FUNC_DELGROUPMEM:   <gname> <member> delgroupmem <flag>
 *   Remove a user from a group.  Returns 0 on success, 1 if the group
 *   does not exists or the user was not a member of it.
 */

void func_delgroupmem()
{
#if NOPWEDIT
    die("Changing group membership not enabled in this installation");
#else
    char *mem= pop_string();
    char *gname= pop_string();

    if (!bdict(VAR_AMADM))
    	die("Only conference administrator may remove users from groups");

    push_integer(delgroupmem(gname,mem));
    free(mem);
    free(gname);
#endif /*NOPWEDIT*/
}
